#######################################################
#MULTIPLE REGRESSION
#######################################################

# Read in anesthesia data

anesth<-read.table("anesth.dat",header=T)
attach(anesth)
# Use the lm() function to regress the response variable on 
# the two explanatory variables
summary(lm(Depress~Duration+Trauma))
help(lm)
LM1<-lm(Depress~Duration+Trauma)
res<-residuals(LM1)
hist(res)
shapiro.test(res)
plot(LM1)

# Data for 25 patients with cystic fibrosis
# PEmax is a measure of the malnutrition of the patients
cystic<-read.table("cystic.dat",header=T)
attach(cystic)

# Use the pairs function to produce a variety of scatterplot matrices
win.graph()
pairs(cystic[,-c(1,3)])

# Produce a scatterplot to check relationship between Sex and BMP
win.graph()
plot(Sex,BMP,axes=F)
axis(1,at=c(0,1),labels=c("Males","Females"))
axis(2)


# Enhanced scatterplot matrix
win.graph()
pairs(cystic[,-c(1,3)],
   panel=function(x,y) {points(x,y)
      abline(lm(y~x))
      lines(lowess(x,y),lty=2)
   }
)

# Now regress PEmax on all the explanatory variables using the lm function
cystic.lm<-lm(PEmax~Age+Sex+Height+Weight+BMP+FEV+RV+FRC+TLC)
summary(cystic.lm)

#stepwise selection -> manually

# Examples of forward selection and backward elimination
lm1<-lm(PEmax~.,data=cystic)
summary(lm1)
slmf<-step(lm1,method="forward")
summary(slmf)
#compare
slmb<-step(lm1,method="backward")
summary(slmb)


# Fit model chosen by forward and backward methods
# and check assumptions by examining some diagnostics
cystic.lm1<-lm(PEmax~Weight+BMP+FEV+RV)
win.graph()
par(mfrow=c(2,2))
# Use plot() with ask=T and select number of plot required
# (here 2,5,6) type 0 to finish
plot(cystic.lm1,ask=T)
# Regression results
summary(cystic.lm1)

