#-------------------------------------------------------------------
#  
#  Exercise on unmatched case-control study
#  The Danish malignant melanoma study (Osterlind et al. 1988)
#  Friday 27 May at 10.30-12.30 
#  
#  Solution sheet (Esa Lr 28 May)
#  http://staff.pubhealth.ku.dk/~bxc/SPE/data/melanoma05.R
#--------------------------------------------------------------------

# Two misprints were found in the exercise sheet:
# - item 7. 'agr' should be 'age.cat' (like later and in the solution)
# - item 15. A tilde '~' is missing from the model formula btw two '.'s

# 1. Read and view the data frame & load the Epi package:
library( Epi )
# The following presumes that the data file is in the working
# directory. Otherwise the full directory path must be given.
# From the web: "http://www.biostat.ku.dk/~bxc/SPE/data/melanoma.dat"
mm <- read.table( file="../data/melanoma.dat", header=T, na.strings="." )
mm <- read.table( file="http://www.biostat.ku.dk/~bxc/SPE/data/melanoma.dat", 
                  header=T, na.strings="." )
mm[1:10, ]

# 2. Structure of the data frame and univariate summaries
str(mm)
summary(mm)

# 3. Checking how many completely observed subjects there are and 
#    trimming the data frame to include only those
sum( complete.cases(mm) )
mm <- subset(mm, complete.cases(mm) == T)

# 4. Stem-and-leaf diagrams
stem(mm$nvlarge)
stem(mm$nvsmall)

# 5. Conversion to factors according to the given coding of the variables
mm$sex  <- factor( mm$sex,  labels=c(" male"," female") )
mm$skin <- factor( mm$skin, labels=c(" dark"," medium"," light") )
mm$hair <- factor( mm$hair, labels=c(" dark"," light brown"," blond"," red") )
mm$eyes <- factor( mm$eyes, labels=c(" brown"," gray/green"," blue") )

# 6. Coding of freckles reversed; conversion to a factor (unordered)
table( 
mm$freckles,   factor( 4 - mm$freckles, labels=c(" none"," some"," many") ) )
mm$freckles <- factor( 4 - mm$freckles, labels=c(" none"," some"," many") )

# 7. Categorizing age into age groups and giving labels to them
mm$age.cat <- cut(mm$age, br=c(20, 40, 50, 60, 70, 85), 
                          include.lowest=T, right=F)
levels(mm$age.cat) <- c( " 20-39", " 40-49", " 50-59", " 60-69", " 70-84" ) 
# Alternatively
# mm$age.cat <- cut(mm$age, br = c(20, 40, 50, 60, 70, 85), 
#              labels = c( " 20-39", " 40-49", " 50-59", " 60-69", " 70-84" ),  
#                include.lowest=T, right=F)
table(mm$age.cat,mm$cc)

# 8. Overview and attachment of the data frame
str( mm )
summary( mm )
attach( mm )

# 9. Simple two-way tables of counts and percentages
table( skin, cc )
pctab( table( skin, cc ), margin=1, dec=1 )

table( hair, cc )
pctab( table( hair, cc ), margin=1, dec=1 )
     
table( eyes, cc )
pctab( table( eyes, cc ), margin=1, dec=1 )

table( freckles, cc  )
pctab( table( freckles, cc ), margin=1, dec=1 )

# 10. Odds ratio for freckles: "many" vs. "none"
tabfr <- table( freckles, cc)
# The twoby2() function assumes that "exposed" is indexed before "unexposed"
# and "cases" before "controls" in its argument table, so therefore we
# reverse the indexes when extracting the rows for "many" and "none" to it
tabfr31 <- tabfr[ c(3,1), c(2,1)] ; tabfr31
twoby2 ( tabfr31, alpha=0.1 )

# 11. Estimation of the same OR with logistic regression
options(show.signif.stars=F)
m.fr <- glm( cc ~ freckles, family=binomial, data=mm )
summary(m.fr)
ci.lin(m.fr,Exp=TRUE) 
# Only points estimates of the logistic coefficients (1st col) and their  
# confidence limits (5th and 6th) are meningful to transform on the OR scale:
round( exp( ci.lin(m.fr, alpha=0.1)[ , c(1,5,6)]  ), 3)

# 12. Crude odds ratio for "red, blond or light brown" vs. "dark" hair 
#     Levels 2, 3, and 4 are collapsed into one category first
mm$hair2 <- Relevel(mm$hair, list( 1, 2:4 ) )
attach(mm)
table( hair2, cc )
twoby2( table( hair2, cc) ) 
 
# 13. The same by logistic regression
m.h2 <- glm( cc ~ hair2, family=binomial, data=mm )
round(summary( m.h2 )$coef, 4)
round( exp( ci.lin(m.h2, alpha=0.1)[ , -(2:4) ]  ), 3)

# 14. 2x2 table of hair colour and case-control status stratified by age 
ftable(age.cat, hair2, cc )
# The stratified percentage tables are a bit tricky to construct
round( ftable( pctab( table(age.cat, hair2, cc), margin=2 ) ) , 1)

# 15. Mantel-Haenszel estimation
# First construct tables for cases and controls by age and hair colour
cases.h2a <- table(hair2, age.cat, cc)[  , , 2] 
cases.h2a
ctrls.h2a <- table(hair2, age.cat, cc)[  , , 1] 
ctrls.h2a
# MH-estimation is based on the above tables, and levels 2 and 1
# of the first dimension of them are contrasted
mh.h2a <- mh(cases.h2a, ctrls.h2a, levels = c(2,1) )
str(mh.h2a)   # view the structure of the mh object
cbind( mh.h2a$ratio, mh.h2a$cl.lower, mh.h2a$cl.upper)

# 16. Age-adjustment by a model
m.h2a <- update(m.h2, . ~ . + age.cat )
round(summary( m.h2a )$coef, 4)
round( exp( ci.lin(m.h2a, alpha=0.1)[ , c(1,5,6)] ), 3)

# 17. Numbers of small and large naevi categorized
mm$nvlar3 <- cut(mm$nvlarge, br=c(0,1,2, 15 ), 
              include.lowest=T, right=F)
mm$nvsma4 <- cut(mm$nvsmall, br=c(0,1,2,5, 50 ), 
              include.lowest=T, right=F)
attach(mm)
# Check how much small and large naevi are associated
table(nvsma4, nvlar3)
round(pctab( table( nvsma4, nvlar3), 2), 1)

# 18. The effects of numbers of small and large naevi
m.nvsma <- glm(cc ~ sex + age.cat + nvsma4, # small only
          family=binomial, data= mm)
round(summary( m.nvsma)$coef, 4)
round( exp( ci.lin(m.nvsma, alpha=0.1)[ , c(1,5,6)]  ), 3)

m.nvlar <- glm(cc ~ sex + age.cat + nvlar3,  # large only
          family=binomial, data= mm)
round(summary( m.nvlar )$coef, 4)
round( exp( ci.lin(m.nvlar, alpha=0.1)[ , -(2:4)]  ), 3)

m.nvboth <- update(m.nvsma, . ~ . + nvlar3)  # both kinds of naevi
round(summary( m.nvboth )$coef, 4)
round( exp( ci.lin(m.nvboth, alpha=0.1)[ , c(1,5,6)]  ), 3)

# 19. Adding freckles to the model
m.nvf <- update(m.nvboth, . ~ . + freckles ) 
round(summary( m.nvf )$coef, 4)
round( exp( ci.lin(m.nvf, alpha=0.1)[ , -(2:4)]  ), 3)
anova(m.nvf, m.nvboth)  # deviance test for addition of freckles

# 20. Numeric freckles score created and linear trend tests performed
mm$fscore <- as.numeric(mm$freckles) 
attach(mm) 
table(freckles, fscore)

m.nvfsc <- update(m.nvboth, . ~ . + fscore)
round(summary( m.nvfsc )$coef, 4)
2*( 1 - pnorm( abs( 5.94) ) )     # P-value from the Wald statistic
round( exp( ci.lin(m.nvfsc, alpha=0.1)[ , c(1,5,6)]  ), 3)
anova(m.nvfsc, m.nvboth)
1 - pchisq(35.72, df=1)      # P-value from the deviance statistic

# 21. Return freckles as unordered, and add hair2 & skin
m.final <- update( m.nvf, . ~ . + hair2 + skin )
round(summary( m.final )$coef, 4)
round( exp( ci.lin(m.final, alpha=0.1)[ , c(1,5,6)]  ), 3)

# 22. Plotting the OR estimates and their CIs
#     two versions, simple and with more sophistication
pdf( "../graph/mel05-est.pdf", height=8, width=7 )
par( mfrow=c(2,1), mar=c(3,2,1,1) )
plotEst( exp( ci.lin( m.final )[-(1:6), -(2:4)] ), xlog=T )
plotEst( exp( ci.lin( m.final )[-(1:6), -(2:4)] ),
         txt=c("Small naevi: 1", "2-5", "5+",
               "Large naevi: 1", "2+",
               "Freckles: Some", "Many",
               "Hair: light/blond/red",
               "Skin: medium", "light" ),         
         xlog=T, grid=T, vref=1 )
